#!/usr/bin/python
# -*- coding: utf-8 -*-
from gettext import gettext as _

import deskbar.interfaces.Module
import deskbar.interfaces.Match
import deskbar.interfaces.Action
from deskbar.defs import VERSION
from deskbar import DEFAULT_RESULTS_PER_HANDLER

import os
import gtk
import gtk.glade

import ldap

HANDLERS = ["AnnuaireAuFHandler"]

class AnnuaireAuFHandler(deskbar.interfaces.Module):

    INFOS = {
        "icon": deskbar.core.Utils.load_icon('stock_directory-server'),
        "name": _("Annuaire AuF"),
        "description": _("Recherche dans l'annuaire AuF"),
        "version": VERSION + '.1'
    }

    def __init__(self):
        deskbar.interfaces.Module.__init__(self)
        self._ldap = LdapServer(server='ldap.auf', base='o=AuF')

    def initialize(self):
        try:
            self._ldap.connect()
        except:
            pass

    def stop(self):
        pass

    def query(self, query):
        if not query.startswith("auf "): return
        real_query = query[len("auf "):]
        if len(real_query) < 3: return

        aux = []
        try:
            aux = self._ldap.query(real_query, DEFAULT_RESULTS_PER_HANDLER)
        except:
            title = _("problème d'accès au serveur")
            name = "%s : %s" % (self.INFOS['name'], title)
            match = deskbar.interfaces.Match(name=name, category="people")
            match.add_action( ErrorMessageAction(name) )
            self._emit_query_ready(query, [ match ] )
            return
        if aux == []:
            title = _("aucun résultat correspondant à la recherche")
            name = "%s : %s" % (self.INFOS['name'], title)
            match = deskbar.interfaces.Match(name=name, category="people")
            match.add_action( ErrorMessageAction(name) )
            self._emit_query_ready(query, [ match ] )
            return

        result = []
        for entry in aux:
            title = entry[1]['cn'][0] + ' <' + entry[1]['mail'][0] + '>'
            name = "%s : %s" % (self.INFOS['name'], title)
            match = deskbar.interfaces.Match(name=name, category="people")
            match.add_action( AnnuaireAuFAction(name, entry[1]) )
            result.append(match)
        self.set_priority_for_matches(result)
        self._emit_query_ready(query, result)

class ErrorMessageAction(deskbar.interfaces.Action):

    def get_verb(self):
        return "%(name)s"

    def activate(self, text=None):
        pass

    def skip_history(self):
        return True

class AnnuaireAuFAction(deskbar.interfaces.Action):
    """
    Affiche une fenêtre avec les informations collectées depuis le LDAP.
    """

    def __init__(self, name, data):
        deskbar.interfaces.Action.__init__(self, name)
        self._data = data

    #def get_icon(self):
    #    return "gtk-open"

    def get_verb(self):
        return "%(name)s"

    def activate(self, text=None):
        # champs à afficher dans l'ordre, sous forme de (nom, clé, index)
        # si index < 0 alors on affiche toutes les valeurs pour cette clé
        fields = (
            ('cn', "Nom", 0), ('mail', "Adél", 0),
            ('o', "Organisme", 1), ('ou', "Rattachement", 2),
            ('l', "Pays", 0), ('telephoneNumber', "Téléphone", -1),
            ('vacationInfo', "Message d'absence", 0) )
        data = []
        for key,name,index in fields:
            if self._data.has_key(key) and index < len(self._data[key]):
                if index >= 0:
                    data.append( (name, self._data[key][index]) )
                else:
                    data.append( (name, self._data[key][0]) )
                    for value in self._data[key][1:]:
                        data.append( ('', value) )
        ShowInfoWidget(self._name, data).show()

def wordwrap(string, width):
    result = []
    for line in string.splitlines():
        line = line.rstrip()
        while len(line) > width:
            i = line.rfind(' ', 0, width)
            if i < 0:
                result.append(line[:width])
                line = line[width:]
            else:
                result.append(line[:i])
                line = line[i+1:]
        result.append(line)
    return result

class ShowInfoWidget:

    def __init__(self, title, data):
        self._widgets = gtk.glade.XML(os.path.join(deskbar.SHARED_DATA_DIR, "AnnuaireAuF.glade"))
        self._widgets.get_widget("ShowInfoWidget").set_title(title)

        treeView = self._widgets.get_widget("treeView")
        treeView.set_rules_hint(True)

        column1 = gtk.TreeViewColumn('Attribut')
        column2 = gtk.TreeViewColumn('Valeur')
        treeView.append_column(column1)
        treeView.append_column(column2)

        cell = gtk.CellRendererText()
        column1.pack_start(cell, True)
        column2.pack_start(cell, True)
        column1.add_attribute(cell, 'text', 0)
        column2.add_attribute(cell, 'text', 1)

        listModel = gtk.ListStore(str, str)
        for key,value in data:
            if len(value) > 60:
                value = wordwrap(value, 60)
                listModel.append( (key, value[0]) )
                for line in value[1:]:
                    listModel.append( ('', line) )
            else:
                listModel.append( (key, value) )
        treeView.set_model(listModel)

    def show(self):
        self._widgets.get_widget("ShowInfoWidget").show()

class LdapServer:
    _server = ''
    _port = 389
    _protocol = ldap.VERSION3
    _user = ''
    _password = ''
    _base = ''
    _scope = ldap.SCOPE_SUBTREE
    _filter = '(&(|(ou=People)(ou=Vacation))(|(mail=%s*)(cn=%s*)(sn=%s*)))'
    _attribs = None
    _is_connected = False

    def __init__(self, **kvargs):
        for key in kvargs:
            if hasattr(self, '_'+key):
                setattr(self, '_'+key, kvargs[key])

    def connect(self, force=False):
        if not force and self._is_connected: return
        try:
            l = ldap.open(self._server, self._port)
            l.protocol_version = self._protocol
            l.simple_bind(self._user, self._password)
            self._ldap = l
            self._is_connected = True
        except ldap.LDAPError, bind_error:
            self._ldap = None
            self._is_connected = False
            # FIXME: est-ce la bonne façon de remonter l'erreur ?
            raise ldap.LDAPError, bind_error

    def query(self, query, max=25):
        if type(query) == str:
            query = self._filter.count('%s') * (query,)

        self.connect()
        result_id = self._ldap.search_ext(
            self._base, self._scope, self._filter % query,
            self._attribs, 0, None, None, -1, max)

        result_set = []
        while True:
            try:
                result_type, result_data = self._ldap.result(result_id, 0, 20)
            except ldap.SIZELIMIT_EXCEEDED, s_e:
                break
            if result_data == []:
                break
            if result_type == ldap.RES_SEARCH_ENTRY:
                result_set.append(result_data[0])
        return result_set

# test
if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        search = sys.argv[1]
    else:
        search = "andr"
    server = LdapServer(server='ldap.auf', base='o=AuF')
    aux = server.query(search, 2)
    entry = aux[0][1]
    title = entry['cn'][0] + ' <' + entry['mail'][0] + '>'
    data = []
    for key in entry:
        data.append( (key, entry[key][0]) )
        for value in entry[key][1:]:
            data.append( ('', value) )
    w = ShowInfoWidget(title, data)
    w.show()
    try:
        gtk.main()
    except KeyboardInterrupt:
        pass

